{-# LINE 30 "Traversal.lhs" #-}
  {-#  LANGUAGE RankNTypes  #-}
 
  module Algos.Graphs.Traversal ( traverse
                                , traverse'
                                , traverseFrom
                                , traverseFrom'
                                , VertexSet(..)
                                ) where
 
  import Algos.Graphs.Forest (Forest(..),Tree,UpTree,toForestIndexByIx)
  import Algos.Graphs.Graph (AdjList(..),V(..),E(..),GraphStructure(..))
  import Control.Applicative ((<$>))
  import Control.Monad.ST (ST,runST)
{-# LINE 67 "Traversal.lhs" #-}
  traverse   ::  VertexSet vs => (forall s . Int -> ST s (vs s)) -> AdjList v vl el -> Forest V E
  traverse'  ::  VertexSet vs => (forall s . Int -> ST s (vs s)) -> AdjList v vl el -> UpTree V E
{-# LINE 80 "Traversal.lhs" #-}
  class VertexSet vs where
    add     ::  vs s -> V -> [(E,V)] -> ST s ()
    remove  ::  vs s -> ST s (Maybe (V, [(E,V)]))
{-# LINE 96 "Traversal.lhs" #-}
  traverse makeSet = toForestIndexByIx vIx . traverse' makeSet
{-# LINE 106 "Traversal.lhs" #-}
  traverse' makeSet g = runST (traverseM makeSet (adjGraph g))
 
  traverseM :: VertexSet vs => (Int -> ST s (vs s)) -> GraphStructure -> ST s (UpTree V E)
{-# LINE 113 "Traversal.lhs" #-}
  traverseM makeSet g = do  s <- makeSet (gNumVertices g)
                            concat <$> mapM (traverseFromVertexM s) (gVertices g)
 
  traverseFromVertexM      ::  VertexSet vs => vs s -> V -> ST s (UpTree V E)
  traverseFromVertexM s v  =   add s v [] >> loop
    where  loop = do  nxt <- remove s
                      case nxt of
                        Nothing        ->  return []
                        Just p@(v,as)  ->  do  mapM_ (\e -> add s (eHead e) ((e,v):as)) (vOutEdges v)
                                               (p:) <$> loop
{-# LINE 137 "Traversal.lhs" #-}
  traverseFrom               ::  VertexSet vs => (forall s . Int -> ST s (vs s)) -> AdjList v vl el -> V -> Tree V E
  traverseFrom makeSet g v   =   head . trees $ toForestIndexByIx vIx $ traverseFrom' makeSet g v
 
  traverseFrom'              ::  VertexSet vs => (forall s . Int -> ST s (vs s)) -> AdjList v vl el -> V -> UpTree V E
  traverseFrom' makeSet g v  =   runST $ do  s <- makeSet (gNumVertices $ adjGraph g)
                                             traverseFromVertexM s v
